@@ -314,6 +314,14 @@ fr_command_cfile_add (FrCommand  *comm,
 		fr_process_add_arg (comm->process, compressed_filename);
 		fr_process_end_command (comm->process);
 	}
+	else if (_g_mime_type_matches (archive->mime_type, "application/zstd")) {
+		fr_process_begin_command (comm->process, "zstd");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "--");
+		fr_process_add_arg (comm->process, filename);
+		fr_process_end_command (comm->process);
+		compressed_filename = g_strconcat (filename, ".zst", NULL);
+	}
 	else {
 		g_warning ("Unhandled mime type: '%s'", archive->mime_type);
 		g_warn_if_reached ();
@@ -478,6 +486,14 @@ fr_command_cfile_extract (FrCommand  *comm,
 		fr_process_add_arg (comm->process, uncompr_file);
 		fr_process_end_command (comm->process);
 	}
+	else if (_g_mime_type_matches (archive->mime_type, "application/zstd")) {
+		fr_process_begin_command (comm->process, "zstd");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, "-d");
+		fr_process_add_arg (comm->process, temp_file);
+		fr_process_end_command (comm->process);
+	}
 
 	/* copy uncompress file to the dest dir */
 
@@ -521,6 +537,7 @@ const char *cfile_mime_type[] = { "application/x-gzip",
 				  "application/x-lzop",
 				  "application/x-rzip",
 				  "application/x-xz",
+				  "application/zstd",
 				  NULL };
 
 
@@ -582,6 +599,10 @@ fr_command_cfile_get_capabilities (FrArchive  *archive,
 		if (_g_program_is_available ("lz4", check_command))
 			capabilities |= FR_ARCHIVE_CAN_READ_WRITE;
 	}
+	else if (_g_mime_type_matches (mime_type, "application/zstd")) {
+		if (_g_program_is_available ("zstd", check_command))
+			capabilities |= FR_ARCHIVE_CAN_READ_WRITE;
+	}
 	return capabilities;
 }
 
@@ -610,6 +631,8 @@ fr_command_cfile_get_packages (FrArchive  *archive,
 		return PACKAGES ("rzip");
 	else if (_g_mime_type_matches (mime_type, "application/x-lz4"))
 		return PACKAGES ("lz4");
+	else if (_g_mime_type_matches (mime_type, "application/zstd"))
+		return PACKAGES ("zstd");
 
 	return NULL;
 }
